from model.user import User
from error import Missing, Duplicate

# (no hashed password checking in this module)
fakes = [
    User(name="kwijobo",
         hash="abc"),
    User(name="ermagerd",
         hash="xyz"),
    ]

def find(name: str) -> User | None:
    for e in fakes:
        if e.name == name:
            return e
    return None

def check_missing(name: str):
    if not find(name):
        raise Missing(msg=f"Nie ma użytkownika {name}")

def check_duplicate(name: str):
    if find(name):
        raise Duplicate(msg=f"Użytkownik {name} już istnieje")

def get_all() -> list[User]:
    """Zwraca wszystkich użytkowników"""
    return fakes

def get_one(name: str) -> User:
    """Zwraca pojedynczego użytkownika"""
    check_missing(name)
    return find(name)

def create(user: User) -> User:
    """Dodaje użytkownika"""
    check_duplicate(user.name)
    return user

def modify(name: str, user: User) -> User:
    """Częściowo modyfikuje użytkownika"""
    check_missing(name)
    return user

def delete(name: str) -> None:
    """Usuwa użytkownika"""
    check_missing(name)
    return None
